//Modify to work with digital temperature probe.
//19August2017: modify so that after bringing to boil, it does not resume heating
//after beeping unless one has flipped the switch (let it continue to cool). This
//lets me remove pot from cooker shell for fast cooling. 

//Using old RBBB, board = "Duemilanove"
//Processor = ATmega328
//Port = /dev/usbserial-XXXX where XXXX is like AH02IXBZ

#include <OneWire.h>
#include <DallasTemperature.h>

// Data wire is plugged into digital pin 2 on the Arduino
#define ONE_WIRE_BUS 2
 
// Setup a oneWire instance to communicate with any OneWire devices 
// (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
 
// Pass our oneWire reference to Dallas Temperature.
DallasTemperature sensors(&oneWire);

//assign pins     
//const int analogPin = 0;           // thermistor
const int yogpin = 5;              // switch
const int relaypin = 13;
const int yellowled = 10;
const int buzzpin = 11;


// set temperature ranges - now in deg C
const int minboil = 86;
const int maxboil = 90; //+3 overshoot
// Reduce a degree or so 27Jan2018
const int minbrew = 41; //same as max because <low, >max
const int maxbrew = 42;   //+3 overshoot
const int cooled = 47;   
const int disconnected = -100;   // less than this (or negative this?)


// Variables will change :
int temper = 0;                    //analog input value
int switchval = HIGH;              //LOW = yogurt added; read on each loop
int neverboiled = true;            //check

// use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = 0;        // will store time

//Max interval = 2^32-1 ms or about 1193 hours

//1 minute = 60000 ms

// constants won't change :
const long heattime = 2400000;       // 40 minutes
const long startup = 120000;          // 2 minutes

void setup() {
  Serial.begin(9600);      // open the serial port at 9600 bps: 
// set the digital pins as output:
  Serial.println("Setup"); 
// Start up the library
  sensors.begin();
// Other pins...  
  pinMode(relaypin, OUTPUT);
  pinMode(yellowled, OUTPUT);
  pinMode(buzzpin, OUTPUT);
  pinMode(yogpin, INPUT); 
}


void loop()
{
  unsigned long currentMillis = millis();
  unsigned long minutes = currentMillis/60000;      // integer divide so..
  
  Serial.println(temper);            // display its value


// This block taken from the temperature probe demo program
// call sensors.requestTemperatures() to issue a global temperature
// request to all devices on the bus
//  Serial.print(" Requesting temperatures...");
  sensors.requestTemperatures(); // Send the command to get temperatures
//  Serial.println("DONE");
    
  temper = sensors.getTempCByIndex(0);

    // You can have more than one IC on the same bus. 
    // 0 refers to the first IC on the wire
   delay(300);


   // startup checks
    if (currentMillis < startup) {             
 //      Serial.println("startup");
       switchval = digitalRead(yogpin);  // check yogurt-added switch

      // check temperature probe resistance
         if(temper < disconnected){  
  
           Serial.println("disconnected probe");
           bip(buzzpin); 
       }

      // if yogurt switch already on
          if(switchval == LOW){  // beep if switch thrown
          Serial.println("yogpin LOW (yog added?)");
           bip(buzzpin);
           }

    }  //end if startup

    
// heattime = time to heat and hold for a bit at high temp
    if ( (currentMillis < heattime) && (neverboiled == true) ) {
       digitalWrite(yellowled, HIGH);          
//      Serial.println("heat time");
//      Serial.println("\n minutes: ");
//      Serial.println(minutes);
       
       if(temper < minboil){
         digitalWrite(relaypin, HIGH);  // heater on 
       }
       if(temper > maxboil){
         digitalWrite(relaypin, LOW);   // heater off
         neverboiled = false;
         Serial.print("\n max: ");
         Serial.print(temper); 
       }

     }  //end if < heattime


//Execute only if heattime is over:
     else {
           digitalWrite(yellowled, LOW); // heat time is over
           digitalWrite(buzzpin, LOW);   // buzzer off unless otherwise noted
           switchval = digitalRead(yogpin);
//         Serial.println("cool time");
//         Serial.println("\n minutes: ");
//         Serial.println(minutes);
//         Serial.println(switchval);    // HIGH (not added) or LOW (added)

           if(temper < cooled){                  // one could add yogurt now
               if(switchval == HIGH){            // beep if switch not thrown
//                  Serial.println("yogpin HIGH");
                   bip(buzzpin);
                   bip(yellowled);               //blink yellow LED also
                   }
           }

//19Aug2017: Turn on heat, if switch was thrown
            if((temper < minbrew) && (switchval == LOW) ){
                  digitalWrite(relaypin, HIGH);  // heater on 
  
                 }                
           if(temper > maxbrew){
                  digitalWrite(relaypin, LOW);   // heater off
                }    

   
           if(neverboiled == true){    // never got up to boil temperature...
              bip(buzzpin);
              Serial.println("never boiled");
           }
      }  //end else if heattime over
       
  

 }  //end main loop


//Bip: Briefly turn pin HIGH
void bip(int pin) {
         digitalWrite(pin, HIGH);
         delay(300);
         digitalWrite(pin, LOW);
         delay(400);
}




